import { currentAccount, getChain } from "../utility/wallet.js";
import { fromWei } from "../utility/ethers.js";
import { ERC20 } from "../contract/ERC20.js";
import { BaseElement } from "@ohlabs/js-core/utility/element/base-element.js";
import { tokenBalanceTemplate } from "../template/token-balance-template.js";
import { tokenConfig } from "@ohlabs/configuration/token.js";

class TokenBalance extends BaseElement {  

    static get observedAttributes() {
        return ['token-address'];
    }

    get address() { return this.getAttribute('token-address'); }
    set address(value) { this.setAttribute('token-address', value); }

    token;
    tokenIcon;
    tokenBalance;

    userConnectedCallback;
    userDisconnectedCallback;

    constructor(){ 
        super();
    }

    async connectedCallback() {
        console.log("connected");
        super.shadowRoot = this.attachShadow({ mode: "open" });
        await this.loadTemplate();
    }

    async loadTemplate(){
        if(!this.loaded){
            super.shadowRoot.innerHTML = tokenBalanceTemplate;
            super.loadCss();

            this.tokenIcon = this.root.getElementById('tokenIcon');
            this.tokenBalance = this.root.getElementById('tokenBalance');

            window.addEventListener('onConnected', event => this.connected(event.detail));
            window.addEventListener('onDisconnected', this.disconnected.bind(this));
            
            await this.load();

            super.loaded = true;
        }
    }

    async load(){
        if(currentAccount && this.address){
            this.token = new ERC20(this.address);

            let chain = await getChain();
            let tokenName = await this.token.name();

            let tokenSettings = tokenConfig.test[chain.name][tokenName];
            if(tokenSettings){
                this.tokenIcon.src = tokenSettings.logoURI
            }
            else{
                this.tokenIcon.src = '/resources/Bubble Desat.png';
                this.tokenIcon.classList.add('unknowntoken');
            }

            await this.loadBalance();
        }
    }

    async loadBalance(){
        if(currentAccount && this.address){
            this.tokenBalance.innerText = fromWei(await this.token.balanceOf(currentAccount) ?? 0);
        }
    }

    async connected(account){
        console.log(`token balance detected connection to ${account}`);

        await this.load();
        if(this.userConnectedCallback) { this.userConnectedCallback(); }
    }

    disconnected(){
        console.log("token balance detected disconnection");
        if(this.userDisconnectedCallback) { this.userDisconnectedCallback(); }
    }
}

customElements.define('ohl-token-balance', TokenBalance);

export { TokenBalance }