import { currentAccount, nativeBalance, getChain } from "../utility/wallet.js";
import { fromWei } from "../utility/ethers.js";
import { BaseElement } from "@ohlabs/js-core/utility/element/base-element.js";
import { profileTemplate } from "../template/profile-template.js";
import { tokenConfig } from "@ohlabs/configuration/token.js";
import { getProfile } from "../utility/profile.js";

class UserProfile extends BaseElement {  

    static get observedAttributes() {
        return ['user-tokens'];
    }

    get userTokens() { return this.getAttribute('user-tokens'); }
    set userTokens(value) { this.setAttribute('user-tokens', value); }

    userName;
    avatar;

    nativeBalance;

    userText;
    addressText;
    avatarIcon;

    nativeBalanceText;
    userTokenDisplay;

    content;
    initializeContent;

    userConnectedCallback;
    userDisconnectedCallback;

    constructor(){ 
        super();
    }

    connectedCallback() {
        console.log("connected");
        super.shadowRoot = this.attachShadow({ mode: "open" });
        this.loadTemplate();
    }

    loadTemplate(){
        if(!this.loaded){
                super.shadowRoot.innerHTML = profileTemplate;
                super.loadCss();

                this.userText = this.root.getElementById('userText');
                this.addressText = this.root.getElementById('addressText');
                this.avatarIcon = this.root.getElementById('avatarIcon');
           
                this.nativeBalanceText = this.root.getElementById('nativeBalanceText');
                this.userTokenDisplay = this.root.getElementById('userTokenDisplay');

                window.addEventListener('onConnected', event => this.connected(event.detail));
                window.addEventListener('onDisconnected', this.disconnected.bind(this));

                if(this.content){ this.loadContent(); }

                super.loaded = true;
                console.log("Profile loaded");
        }
    }

    loadContent() {
        console.log("loading content");
        this.root.getElementById('content').innerHTML = this.content;
        this.initializeContent();
    }

    async loadBalances(){
        if(currentAccount)
        {
            this.nativeBalance = await nativeBalance();
            this.nativeBalanceText.innerText = fromWei(this.nativeBalance);

            if(this.userTokens)
            {
                let chain = await getChain();
                this.userTokenDisplay.innerHTML = this.userTokens.split(',')
                                                                .map(tokenName => {
                                                                    return `<ohl-token-balance css="css/site.css" token-address="${tokenConfig.test[chain.name][tokenName.trim()].address}"></ohl-token-balance>`;
                                                                })
                                                                .join('\n');
            }
        }
    }

    async connected(account){
        console.log(`profile detected connection to ${account}`);
        this.addressText.innerText = account;

        let profile = getProfile();
        this.userText.innerText = profile ? profile.username : 'USER';
        this.avatarIcon.src = profile ? profile.avatar : '/resources/BubbleDesat.png'

        await this.loadBalances();

        if(this.userConnectedCallback) { this.userConnectedCallback(); }
    }

    disconnected(){
        console.log("profile detected disconnection");
        if(this.userDisconnectedCallback) { this.userDisconnectedCallback(); }
    }
}

customElements.define('ohl-profile', UserProfile);

export { UserProfile }